﻿using System.Web.Security;
using System.Web.DynamicData;
using System.Linq.Expressions;
using System.Linq;
using System.Reflection;
using System.Globalization;
using System.ComponentModel;
using System.Data.Linq;

namespace System.Web.Mvc {
    public static class DynamicMvcExtensionMethods {
        public static IQueryable GetQuery(this DynamicMetaTable metaTable, DataContext dataContext) {
            return (IQueryable)metaTable.DataContextProperty.GetValue(dataContext, null);
        }

        // REVIEW: Temporary.  Need to move to DynamicMetaTable
        internal static DynamicMetaMember TryFindColumn(this DynamicMetaTable metaTable, string memberName) {
            DynamicMetaMember member = null;
            try {
                // REVIEW: need an overload that doesn't throw!
                member = metaTable.FindColumn(memberName);
            }
            catch { }
            return member;
        }

        // REVIEW: temporary method until Mvc gets into the main code base
        public static BaseMetadataAttribute GetMetaDataAttribute(this DynamicMetaMember metaMember, Type attribType) {
            AttributeCollection attribs = TypeDescriptor.GetAttributes(metaMember.MetaTable.EntityType);

            foreach (Attribute attrib in attribs) {
                if (!attribType.IsAssignableFrom(attrib.GetType()))
                    continue;

                var mdAttrib = (BaseMetadataAttribute)attrib;

                if (!mdAttrib.FieldName.Equals(metaMember.Name, StringComparison.OrdinalIgnoreCase))
                    continue;

                return mdAttrib;
            }

            return null;
        }

        public static DataContext GetDataContext(this IHttpContext httpContext) {
            const string key = "__DataContext";
            DataContext dataContext = httpContext.Items[key] as DataContext;
            if (dataContext == null) {
                dataContext = (DataContext)DynamicDatabase.TheDatabase.CreateDataContext();

                // REVIEW: hack to work around the fact that CreateDataContext() always returns
                // the same context instance, which is bad
                dataContext = (DataContext)Activator.CreateInstance(dataContext.GetType());

                httpContext.Items[key] = dataContext;
            }
            return dataContext;
        }

        internal static object SingleUntyped(this IQueryable query) {
            var enumerator = query.GetEnumerator();
            enumerator.MoveNext();
            return enumerator.Current;
        }

        private const string StatusMessageKey = "__DynamicStatusMessage";

        public static void SetStatusMessage(this TempDataDictionary tempDataDictionary, string statusMessage) {
            tempDataDictionary[StatusMessageKey] = statusMessage;
        }

        public static string GetStatusMessage(this TempDataDictionary tempDataDictionary) {
            object tmp;
            if (!tempDataDictionary.TryGetValue(StatusMessageKey, out tmp))
                return null;

            return (string)tmp;
        }

        private const string FormErrorInfoKey = "__DynamicFormErrorInfo";

        public static void SetFormErrorInfo(this TempDataDictionary tempDataDictionary, DynamicFormErrorInfo formErrorInfo) {
            tempDataDictionary[FormErrorInfoKey] = formErrorInfo;
        }

        public static DynamicFormErrorInfo GetFormErrorInfo(this TempDataDictionary tempDataDictionary) {
            object tmp;
            if (!tempDataDictionary.TryGetValue(FormErrorInfoKey, out tmp))
                return null;

            return (DynamicFormErrorInfo)tmp;
        }
    }
}
