﻿using System.Web.Security;
using System.Web.DynamicData;
using System.Linq.Expressions;
using System.Linq;
using System.Reflection;
using System.Globalization;
using System.ComponentModel;
using System.Data.Linq;
using System.Web.UI;

namespace System.Web.Mvc {
    public static class DynamicMvcUtil {
        // REVIEW: find better home for this
        public const string ReturnPathQueryStringKey = "returnPath";

        public static bool IsLoggedOnUser(string userName) {
            MembershipUser user = Membership.GetUser();
            if (user == null)
                return false;

            return userName == user.UserName;
        }

        // Build the Linq Where clause that selects the item
        public static IQueryable<T> ApplyPrimaryKeyWhereClause<T>(DynamicMetaTable metaTable, IQueryable<T> query, string id) {
            return (IQueryable<T>)ApplyPrimaryKeyWhereClause(metaTable, (IQueryable)query, id);
        }

        // Build the Linq Where clause that selects the item
        public static IQueryable ApplyPrimaryKeyWhereClause(DynamicMetaTable metaTable, IQueryable query, string id) {

            // REVIEW: handle multiple PK case
            DynamicMetaMember member = metaTable.IdentityMetaMembers[0];

            object value = ChangeType(id, member.Type);

            // e.g. {e}
            var entityParam = Expression.Parameter(metaTable.EntityType, "e");

            // e.g. {e.ProductID}
            var columnExpr = Expression.MakeMemberAccess(entityParam, member.MemberPropertyInfo);

            // e.g. {e.ProductID == 17}
            var equalExpr = Expression.Equal(columnExpr, Expression.Constant(value));

            // e.g. {e => e.ProductID == 17}
            var whereLambda = Expression.Lambda(equalExpr, entityParam);

            var whereCall = Expression.Call(
                typeof(Queryable), "Where",
                new Type[] { metaTable.EntityType },
                query.Expression, whereLambda);

            query = query.Provider.CreateQuery(whereCall);

            return query;
        }

        public static IQueryable<T> ApplyForeignKeyWhereClause<T>(DynamicMetaTable metaTable, DynamicMetaForeignKeyMember fkMember,
            IQueryable<T> query, string id) {
            return (IQueryable<T>)ApplyForeignKeyWhereClause(metaTable, fkMember, (IQueryable)query, id);
        }

        // Build the Linq Where clause that selects the item
        public static IQueryable ApplyForeignKeyWhereClause(DynamicMetaTable metaTable, DynamicMetaForeignKeyMember fkMember,
            IQueryable query, string id) {

            // REVIEW: handle multiple PK case
            DynamicMetaMember idMember = fkMember.ParentMetaTable.IdentityMetaMembers[0];

            object value = ChangeType(id, idMember.Type);

            // e.g. {e}
            var entityParam = Expression.Parameter(metaTable.EntityType, "e");

            // e.g. {e.Category}
            var columnExpr = Expression.MakeMemberAccess(entityParam, fkMember.MemberPropertyInfo);

            // e.g. {e.Category.CategoryId}
            columnExpr = Expression.MakeMemberAccess(columnExpr, idMember.MemberPropertyInfo);

            // e.g. {e.Category.CategoryId == 17}
            var equalExpr = Expression.Equal(columnExpr, Expression.Constant(value));

            // e.g. {e => e.Category.CategoryId == 17}
            var whereLambda = Expression.Lambda(equalExpr, entityParam);

            var whereCall = Expression.Call(
                typeof(Queryable), "Where",
                new Type[] { metaTable.EntityType },
                query.Expression, whereLambda);

            query = query.Provider.CreateQuery(whereCall);

            return query;
        }

        // Build the Linq OrderBy clause that sorts the query
        public static IQueryable<T> ApplyOrderByClause<T>(IQueryable<T> query, string sortColumn) {

            PropertyInfo columnPropInfo = typeof(T).GetProperty(sortColumn);

            // e.g. {e}
            var entityParam = Expression.Parameter(typeof(T), "e");

            // e.g. {e.ProductName}
            var columnExpr = Expression.MakeMemberAccess(entityParam, columnPropInfo);

            // e.g. {e => e.ProductName}
            var orderByLambda = Expression.Lambda(columnExpr, entityParam);

            var orderByCall = Expression.Call(
                typeof(Queryable), "OrderBy",
                new Type[] { typeof(T), columnPropInfo.PropertyType },
                query.Expression, orderByLambda);

            query = query.Provider.CreateQuery<T>(orderByCall);

            return query;
        }

        internal static object ChangeType(string value, Type conversionType) {
            // Convert.ChangeType doesn't support Guid's so handle them separately
            if (conversionType == typeof(Guid)) {
                return new Guid(value);
            }

            return Convert.ChangeType(value, conversionType, CultureInfo.InvariantCulture);
        }

        // Return a comma separated list of the entity's primary keys
        public static string GetEntityKeyString(object entity) {
            DynamicMetaTable metaTable = DynamicDatabase.TheDatabase.GetMetaTable(entity.GetType());
            if (metaTable == null)
                return null;

            string keyString = null;
            foreach (DynamicMetaMember member in metaTable.IdentityMetaMembers) {
                object value = DataBinder.GetPropertyValue(entity, member.Name);
                string stringValue = value.ToString();

                if (keyString == null) {
                    keyString = stringValue;
                }
                else {
                    // REVIEW: we may need to escape commas
                    keyString += "," + stringValue;
                }
            }

            return keyString;
        }
    }
}
