﻿using System.Web.UI;
using System.Globalization;
using System.Web.Mvc;
using System.Collections.Generic;
using System.Web.Util;
using System.Web.DynamicData;
using System.Diagnostics;
using System.Collections.Specialized;

namespace System.Web.Mvc {
    [AspNetHostingPermission(System.Security.Permissions.SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    [AspNetHostingPermission(System.Security.Permissions.SecurityAction.InheritanceDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    public class MvcFieldTemplateUserControlBase : ViewUserControl {
        private DynamicFormFieldErrorInfo _formFieldErrorInfo;
        private bool _triedFormFieldErrorInfo;

        public DynamicMetaMember MetaMember { get; set; }
        public object EntityInstance { get; set; }

        public DynamicViewData DynamicViewData { get; set; }

        public DynamicMetaForeignKeyMember MetaForeignKeyMember {
            get {
                return MetaMember as DynamicMetaForeignKeyMember;
            }
        }

        protected object DataValue {
            get {
                if (EntityInstance == null)
                    return null;

                return DataBinder.GetPropertyValue(EntityInstance, MetaMember.Name);
            }
        }

        protected string DataValueString {
            get {
                // Get the string and preprocess it
                return FormatDataValue(DataValue);
            }
        }

        protected string DataValueEditString {
            get {
                // If there was an error in the previous request, try to get the field's value from
                // there, so the user gets back to what they had typed
                if (FormFieldErrorInfo != null) {
                    return FormFieldErrorInfo.PreviousValue;
                }

                if (DataValue == null)
                    return String.Empty;

                return DataValue.ToString();
            }
        }

        private DynamicFormFieldErrorInfo FormFieldErrorInfo {
            get {
                if (!_triedFormFieldErrorInfo) {
                    if (DynamicViewData.FormErrorInfo != null) {
                        DynamicViewData.FormErrorInfo.FieldErrors.TryGetValue(
                            MetaMember.Name, out _formFieldErrorInfo);
                    }
                    _triedFormFieldErrorInfo = true;
                }

                return _formFieldErrorInfo;
            }
        }

        // Did this field's data in the previous request cause an error
        protected bool HasPreviousErrors {
            get {
                return FormFieldErrorInfo != null && FormFieldErrorInfo.ErrorMessages.Count > 0;
            }
        }

        protected string ForeignKeyPath {
            get {
                // Get the parent entity (e.g. Product.Category)
                object parentEntity = DataBinder.GetPropertyValue(EntityInstance, MetaMember.Name);

                // If we couldn't get one, don't create a link
                if (parentEntity == null)
                    return String.Empty;

                return DynamicViewData.MakeUrl("show", parentEntity);
            }
        }

        protected string ChildrenPath {
            get {
                object dataItem = EntityInstance;
                if (dataItem == null)
                    return String.Empty;

                var column = (DynamicMetaChildrenMember)MetaMember;

                string path = Url.Action("list", column.ChildMetaTable.EntityType.Name);

                // REVIEW: temporary implementation until we merge this into the moan code base.
                // Does not currently work with multiple pks
                var pkColumn=column.MetaTable.IdentityMetaMembers[0];
                object value = DataBinder.GetPropertyValue(dataItem, pkColumn.Name);
                return String.Format("{0}?{1}={2}", path, column.MetaMemberInOtherTable.Name, value);
            }
        }

        // Apply potential HTML encoding and formatting to a string that needs to be displayed
        // This logic is mostly copied from BoundField.FormatDataValue, but omits the old Whidbey behavior path
        // TODO: share some logic with the non MVC case?
        protected virtual string FormatDataValue(object dataValue) {

            string formattedValue = String.Empty;

            if (dataValue != null) {
                string dataValueString = dataValue.ToString();
                string formatting = MetaMember.DataFormatString;
                int dataValueStringLength = dataValueString.Length;

                // If the result is still empty and ConvertEmptyStringToNull=true, replace the value with the NullDisplayText
                if (dataValueStringLength == 0 && MetaMember.ConvertEmptyStringToNull) {
                    dataValueString = MetaMember.NullDisplayText;
                }
                else {
                    // If there's a format string, apply it to the raw data value
                    // If there's no format string, then dataValueString already has the right value
                    if (!String.IsNullOrEmpty(formatting)) {
                        dataValueString = String.Format(CultureInfo.CurrentCulture, formatting, dataValue);
                    }

                    // Optionally HTML encode the value (including the format string, if any was applied)
                    if (!String.IsNullOrEmpty(dataValueString) /*&& MetaMember.HtmlEncode*/) {
                        dataValueString = HttpUtility.HtmlEncode(dataValueString);
                    }
                }

                formattedValue = dataValueString;
            }
            else {
                formattedValue = MetaMember.NullDisplayText;
            }

            return formattedValue;
        }

    }
}
