﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Reflection;

namespace Microsoft.Web.DynamicData.Metadata {
    internal class BuddyMetadataProvider : IMetadataProvider {
        private readonly string _metadataBuddyTypeSuffix = "_Metadata";

        private Dictionary<Type, Type> _buddyTypes = new Dictionary<Type, Type>();

        public List<BaseMetadataAttribute> GetTableMetadata(Type tableType) {
            throw new NotImplementedException();
#if done
            // this is an untested reference implementation for how this sort of thing could be done.
            List<NewBaseMetadataAttribute> results = new List<NewBaseMetadataAttribute>();

            Type buddyType = GetMetadataBuddyType(tableType);
            if (null == buddyType) {
                return results;
            }

            foreach (NewBaseMetadataAttribute attribute in buddyType.GetCustomAttributes(typeof(NewBaseMetadataAttribute), false)) {
                results.Add(attribute);
            }

            return results;
#endif
        }

        public List<BaseMetadataAttribute> GetFieldMetadata(PropertyDescriptor fieldProperty) {
            List<BaseMetadataAttribute> results = new List<BaseMetadataAttribute>();

            Type buddyType = GetMetadataBuddyType(fieldProperty.ComponentType);
            if (null == buddyType) {
                return results;
            }

            foreach (PropertyInfo buddyProperty in buddyType.GetProperties()) {
                if (PropertyMatches(fieldProperty, buddyProperty)) {
                    foreach (BaseMetadataAttribute attribute in buddyProperty.GetCustomAttributes(typeof(BaseMetadataAttribute), false)) {
                        results.Add(attribute);
                    }
                }
            }

            return results;
        }

        private bool PropertyMatches(PropertyDescriptor propertyA, PropertyInfo propertyB) {
            return propertyA.Name.Equals(propertyB.Name, StringComparison.Ordinal) &&
                propertyA.PropertyType == propertyB.PropertyType;
        }

        private Type GetMetadataBuddyType(Type type) {
            Type buddyType;
            if (!_buddyTypes.TryGetValue(type, out buddyType)) {
                // try association attribute
                MetadataClassAttribute attribute = (MetadataClassAttribute) Attribute.GetCustomAttribute(type, typeof(MetadataClassAttribute));
                if (attribute != null && attribute.MetadataClassType != null) {
                    buddyType = attribute.MetadataClassType;
                }

                // try naming pattern
                if (null == buddyType) {
                    buddyType = type.Assembly.GetType(type.FullName + _metadataBuddyTypeSuffix);
                }

                _buddyTypes[type] = buddyType;
            }
            return buddyType;
        }
    }
}
