﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Web.DynamicData;

namespace Microsoft.Web.DynamicData.Metadata {
    internal class OldStyleMetadataTypeDescriptor : CustomTypeDescriptor {
        private Type _type;
        private AttributeCollection _classLevelAttributesFromBase;

        // TODO this code is probably not thread safe
        private bool _recursing = false;

        public OldStyleMetadataTypeDescriptor(Type type, ICustomTypeDescriptor parent)
            : base(parent) {
            _type = type;
        }

        public override AttributeCollection GetAttributes() {
            if (_recursing && _classLevelAttributesFromBase != null) {
                // this special case is meant to handle the situation when we have a "self-type-referencing" class, like:
                // class A {
                //      public A Self;
                // }
                // The reason why this is needed is that by design when TypeDescriptor retrieves the attributes of a given
                // property it also retrieves the attributes of the property's type. In the abovementioned case we find
                // ourselves first getting the attributes on the A class. This looks at all the properties of A and gets
                // their attributes. When we get to the Self property, it once again tries to get the attributes of the A class
                // which causes an infinite recurssion. This special case breaks out of the recurssion.

                return _classLevelAttributesFromBase;
            }

            _recursing = true;
            AttributeCollection attributes = base.GetAttributes();

            // we discard existing old-style attributes
            List<Attribute> attributesToKeep = new List<Attribute>();
            foreach (Attribute attribute in attributes) {
                if (CanKeepClassLevelAttribute(attribute)) {
                    attributesToKeep.Add(attribute);
                }
            }
            // only make new collection if it's needed
            if (attributesToKeep.Count != attributes.Count) {
                attributes = new AttributeCollection(attributesToKeep.ToArray());
            }

            // keep a collection of class-level attributes for the recurssion-breaking special case
            _classLevelAttributesFromBase = attributes;

            // add converted old-style attributes, if any exist
            System.Web.DynamicData.BaseMetadataAttribute[] convertedAttributes = GetConvertedOldStyleAttributes();
            if (convertedAttributes.Length > 0) {
                attributes = AttributeCollection.FromExisting(attributes, convertedAttributes);
            }

            _recursing = false;
            return attributes;
        }

        private System.Web.DynamicData.BaseMetadataAttribute[] GetConvertedOldStyleAttributes() {
            // find all new-style attributes, convert them to old-style attributes and add them to attribute collection.
            List<System.Web.DynamicData.BaseMetadataAttribute> convertedAttributes = new List<System.Web.DynamicData.BaseMetadataAttribute>();

            foreach (PropertyDescriptor propertyDescriptor in TypeDescriptor.GetProperties(_type)) {
                foreach (Attribute attribute in propertyDescriptor.Attributes) {
                    BaseMetadataAttribute newStyleAttribute = attribute as BaseMetadataAttribute;
                    if (null != newStyleAttribute) {
                        System.Web.DynamicData.BaseMetadataAttribute oldStyleAttribute = MetadataAttributeConverter.Instance.ConvertFromNewToOld(propertyDescriptor.Name, newStyleAttribute);
                        convertedAttributes.Add(oldStyleAttribute);
                    }
                }
            }

            return convertedAttributes.ToArray();
        }

        private bool CanKeepClassLevelAttribute(Attribute attribute) {
            return !(attribute is System.Web.DynamicData.BaseMetadataAttribute);
        }
    }
}
