﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Web;
using System.Web.Caching;
using System.Web.Hosting;
using System.Xml;

namespace Microsoft.Web.DynamicData.Metadata {
    internal class XmlMetadataProvider : IMetadataProvider {
        private readonly string _metadataKey = "metadataKey";
        private readonly string _metadataFilePath = "~/metadata.xml";
        private CacheWrapper _cacheWrapper;

        protected virtual CacheWrapper Cache {
            get {
                if (_cacheWrapper == null) {
                    _cacheWrapper = new CacheWrapper(HttpContext.Current.Cache);
                }
                return _cacheWrapper;
            }
        }

        protected virtual string XmlFilePath { get { return HostingEnvironment.MapPath(_metadataFilePath); } }

        protected List<XmlTableMetadataContainer> Metadata {
            get {
                List<XmlTableMetadataContainer> propertyMapping = Cache[_metadataKey] as List<XmlTableMetadataContainer>;
                if (null == propertyMapping) {
                    propertyMapping = LoadMetadata();
                    Cache.Insert(_metadataKey, propertyMapping, new CacheDependency(XmlFilePath));
                }
                return propertyMapping;
            }
        }

        private List<XmlTableMetadataContainer> LoadMetadata() {
            XmlDocument metadataDoc = new XmlDocument();
            metadataDoc.Load(XmlFilePath);
            return XmlMetadataReader.ReadAllMetadata(metadataDoc);
        }

        public List<BaseMetadataAttribute> GetTableMetadata(Type tableType) {
            var container = Metadata.Find(c => c.Name == tableType.Name);
            if (container != null) {
                return new List<BaseMetadataAttribute>(container.TableMetadata);
            }
            return new List<BaseMetadataAttribute>();
        }

        public List<BaseMetadataAttribute> GetFieldMetadata(PropertyDescriptor tableProperty) {
            XmlTableMetadataContainer container = Metadata.Find(c => c.Name == tableProperty.ComponentType.Name);
            if (container != null) {
                List<BaseMetadataAttribute> result;
                if (container.ColumnMetadata.TryGetValue(tableProperty.Name, out result)) {
                    return new List<BaseMetadataAttribute>(result);
                }
            }
            return new List<BaseMetadataAttribute>();
        }

        internal class CacheWrapper {
            private Cache Cache { get; set; }
            public CacheWrapper(Cache cache) {
                Cache = cache;
            }

            public virtual object this[string key] { get { return Cache[key]; } }

            public virtual void Insert(string key, object value, CacheDependency dependency) {
                Cache.Insert(key, value, dependency);
            }
        }
    }
}
