﻿using System;
using System.Collections.Generic;
using System.Xml;

namespace Microsoft.Web.DynamicData.Metadata {
    internal static class XmlMetadataReader {
        public static List<XmlTableMetadataContainer> ReadAllMetadata(XmlDocument document) {
            List<XmlMetadataAttributeHandlerConfigElement> config = GetMetadataHandlerConfiguration(document);
            XmlMetadataAttributeHandler handler = new XmlMetadataAttributeHandler(config);
            return ReadAllMetadata(document, handler);
        }

        internal static List<XmlTableMetadataContainer> ReadAllMetadata(XmlDocument document, XmlMetadataAttributeHandler handler) {
            List<XmlTableMetadataContainer> results = new List<XmlTableMetadataContainer>();
            XmlElement tablesElement = document.DocumentElement.GetChildByName("tables");
            foreach (XmlElement tableElement in tablesElement.GetChildrenByName("table")) {
                string tableName = tableElement.GetAttribute("name");
                List<BaseMetadataAttribute> tableMetadata = GetTableMetadata(tableElement, handler);
                Dictionary<string, List<BaseMetadataAttribute>> columnMetadata = GetColumnMetadata(tableElement, handler );

                XmlTableMetadataContainer container = new XmlTableMetadataContainer() {
                    Name = tableName,
                    ColumnMetadata = columnMetadata,
                    TableMetadata = tableMetadata
                };
                results.Add(container);
            }

            return results;
        }

        private static List<XmlMetadataAttributeHandlerConfigElement> GetMetadataHandlerConfiguration(XmlDocument document) {
            List<XmlMetadataAttributeHandlerConfigElement> config = new List<XmlMetadataAttributeHandlerConfigElement>();

            XmlElement metadataAttributesElement = document.DocumentElement.GetChildByName("metadataAttributes");
            foreach (XmlElement metadataAttributeElement in metadataAttributesElement) {
                string tagPrefix = metadataAttributeElement.GetAttribute("tagPrefix");
                string _namespace = metadataAttributeElement.GetAttribute("namespace");
                string _assembly = metadataAttributeElement.GetAttribute("assembly");

                config.Add(new XmlMetadataAttributeHandlerConfigElement(tagPrefix, _namespace, _assembly));
            }

            return config;
        }

        private static List<BaseMetadataAttribute> GetTableMetadata(XmlElement tableElement, XmlMetadataAttributeHandler handler) {
            List<BaseMetadataAttribute> attributes = new List<BaseMetadataAttribute>();
            
            XmlElement tableMetadataElement = tableElement.GetChildByName("tableMetadata");
            if (null == tableMetadataElement) {
                return new List<BaseMetadataAttribute>();
            }

            return ExtractMetadata(tableMetadataElement, handler);
        }

        private static Dictionary<string, List<BaseMetadataAttribute>> GetColumnMetadata(XmlElement tableElement, XmlMetadataAttributeHandler handler) {
            Dictionary<string, List<BaseMetadataAttribute>> columnMetadata = new Dictionary<string, List<BaseMetadataAttribute>>();

            XmlElement tableColumnsElement = tableElement.GetChildByName("columns");
            if (null == tableColumnsElement) {
                return columnMetadata;
            }

            foreach (XmlElement tableColumnElement in tableColumnsElement.GetChildrenByName("column")) {
                columnMetadata[tableColumnElement.GetAttribute("name")] = ExtractMetadata(tableColumnElement, handler);
            }

            return columnMetadata;
        }

        private static List<BaseMetadataAttribute> ExtractMetadata(XmlElement tableColumnElement, XmlMetadataAttributeHandler handler) {
            List<BaseMetadataAttribute> attributes = new List<BaseMetadataAttribute>();

            foreach (XmlNode node in tableColumnElement.ChildNodes) {
                var tableColumnChildElement = node as XmlElement;
                if (tableColumnChildElement != null) {
                    attributes.Add(handler.CreateAttribute(tableColumnChildElement));
                }
            }

            return attributes;
        }
    }
}
