﻿namespace System.Web.DynamicData {
    public static class DynamicSecurityUtility {
        public static bool AllowRead(object entity) {
            var dynamicEntitySecurity = entity as IDynamicEntitySecurity;

            // If there is no interface, allow access
            if (dynamicEntitySecurity == null)
                return true;

            return dynamicEntitySecurity.AllowRead;
        }

        public static bool AllowWrite(object entity) {
            var dynamicEntitySecurity = entity as IDynamicEntitySecurity;

            // If there is no interface, allow access
            if (dynamicEntitySecurity == null)
                return true;

            return dynamicEntitySecurity.AllowWrite;
        }

        public static void EnsureReadAccess(object entity) {
            if (!AllowRead(entity)) {
                throw new Exception(String.Format("Read access to this object of type {0} is not allowed", entity.GetType()));
            }
        }

        public static void EnsureWriteAccess(object entity) {
            if (!AllowWrite(entity)) {
                throw new Exception(String.Format("Write access to this object of type {0} is not allowed", entity.GetType()));
            }
        }
    }
}
