﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.ComponentModel;
using System.Reflection;
using System.Web;
using System.Web.Compilation;
using System.IO;
using System.Collections.Specialized;
using System.Linq.Expressions;
using System.Data;

namespace System.Web.Mvc {
    public static class ButtonsAndLinkExtensions {

        #region Submit Tags
        /// <summary>
        /// Creates a submit button for your form
        /// </summary>
        /// <param name="buttonText">The text for the button face</param>
        public static string SubmitButton(this HtmlHelper helper, string htmlName) {
            return SubmitButton(helper, htmlName, "Submit", null);
        }

        /// <summary>
        /// Creates a submit button for your form
        /// </summary>
        /// <param name="buttonText">The text for the button face</param>
        public static string SubmitButton(this HtmlHelper helper, string htmlName, string buttonText) {
            return SubmitButton(helper, htmlName, buttonText, null);
        }

        /// <summary>
        /// Creates a submit button for your form
        /// </summary>
        public static string SubmitButton(this HtmlHelper helper) {
            return SubmitButton(helper, "formSubmit", "Submit", null);
        }

        /// <summary>
        /// Creates a submit button for your form
        /// </summary>
        /// <param name="htmlName">Name of the button</param>
        /// <param name="buttonText">The text for the button face</param>
        /// <param name="htmlAttributes">Any attributes you want set on the tag. Use anonymous-type declaration for this: new{class=cssclass}</param>
        /// <returns></returns>
        public static string SubmitButton(this HtmlHelper helper, string htmlName, string buttonText, object htmlAttributes) {
            string buttonFormat = "<input type=submit  name=\"{0}\" id=\"{0}\" value=\"{1}\" {2} />";

            string atts = string.Empty;
            if (htmlAttributes != null)
                atts = htmlAttributes.ToAttributeList();

            return string.Format(buttonFormat, htmlName, buttonText, atts);
        }

        /// <summary>
        /// Creates a submit button for your form using an image
        /// </summary>
        /// <param name="htmlName">Name of the button</param>
        /// <param name="buttonText">The text for the button face</param>
        /// <param name="htmlAttributes">Any attributes you want set on the tag. Use anonymous-type declaration for this: new{class=cssclass}</param>
        /// <returns></returns>
        public static string SubmitImage(this HtmlHelper helper, string htmlName, string imageRelativeUrl) {
            return SubmitImage(helper, htmlName, imageRelativeUrl, null);
        }

        /// <summary>
        /// Creates a submit button for your form using an image
        /// </summary>
        /// <param name="htmlName">Name of the button</param>
        /// <param name="buttonText">The text for the button face</param>
        /// <param name="htmlAttributes">Any attributes you want set on the tag. Use anonymous-type declaration for this: new{class=cssclass}</param>
        /// <returns></returns>
        public static string SubmitImage(this HtmlHelper helper, string htmlName, string imageRelativeUrl, object htmlAttributes) {
            string buttonFormat = "<input type=image  name=\"{0}\" id=\"{0}\" src=\"{1}\" {2} />";

            //TODO: Resolve the imageUrl better

            string atts = string.Empty;
            if (htmlAttributes != null)
                atts = htmlAttributes.ToAttributeList();

            string resolvedUrl = LinkExtensions.ResolveUrl(helper, imageRelativeUrl);


            return string.Format(buttonFormat, htmlName, resolvedUrl, atts);
        }

        #endregion

        #region Button
        /// <summary>
        /// A Simple button you can use with javascript
        /// </summary>
        /// <param name="htmlName">Name of the button</param>
        /// <param name="buttonText">The text for the button face</param>
        /// <param name="onClickMethod">The method or script routine to call when the button is clicked.</param>
        /// <param name="htmlAttributes">Any attributes you want set on the tag. Use anonymous-type declaration for this: new{class=cssclass}</param>
        /// <returns></returns>
        public static string Button(this HtmlHelper helper, string htmlName, string buttonText, string onClickMethod) {
            return Button(helper, htmlName, buttonText, onClickMethod, null);
        }
        /// <summary>
        /// A Simple button you can use with javascript
        /// </summary>
        /// <param name="htmlName">Name of the button</param>
        /// <param name="buttonText">The text for the button face</param>
        /// <param name="onClickMethod">The method or script routine to call when the button is clicked.</param>
        /// <param name="htmlAttributes">Any attributes you want set on the tag. Use anonymous-type declaration for this: new{class=cssclass}</param>
        /// <returns></returns>
        public static string Button(this HtmlHelper helper, string htmlName, string buttonText, string onClickMethod, object htmlAttributes) {
            string buttonFormat = "<input type=button  name=\"{0}\" id=\"{0}\" value=\"{1}\" {2} onclick=\"{3}\" />";

            string atts = string.Empty;
            if (htmlAttributes != null)
                atts = htmlAttributes.ToAttributeList();

            return string.Format(buttonFormat, htmlName, buttonText, atts, onClickMethod);
        }

        /// <summary>
        /// A Simple button you can use with javascript
        /// </summary>
        /// <param name="htmlName">Name of the button</param>
        /// <param name="buttonText">The text for the button face</param>
        /// <param name="onClickMethod">The method or script routine to call when the button is clicked.</param>
        /// <param name="htmlAttributes">Any attributes you want set on the tag. Use anonymous-type declaration for this: new{class=cssclass}</param>
        /// <returns></returns>
        public static string Button<T>(this HtmlHelper helper, Expression<Action<T>> action, string htmlName, string buttonText) where T : Controller {

            string link = LinkExtensions.BuildUrlFromExpression<T>(helper, action);
            return Button(helper, htmlName, buttonText, "location.href='" + link + "'", null);

        }

        /// <summary>
        /// A Simple button you can use with javascript
        /// </summary>
        /// <param name="htmlName">Name of the button</param>
        /// <param name="buttonText">The text for the button face</param>
        /// <param name="onClickMethod">The method or script routine to call when the button is clicked.</param>
        /// <param name="htmlAttributes">Any attributes you want set on the tag. Use anonymous-type declaration for this: new{class=cssclass}</param>
        /// <returns></returns>
        public static string Button<T>(this HtmlHelper helper, Expression<Action<T>> action, string htmlName, string buttonText, object htmlAttributes) where T : Controller {

            string link = LinkExtensions.BuildUrlFromExpression<T>(helper, action);
            return Button(helper, htmlName, buttonText, "location.href='" + link + "'", htmlAttributes);

        }        
        
        #endregion

        #region Image Button
        /// <summary>
        /// A Simple image button you can use with javascript
        /// </summary>
        /// <param name="htmlName">Name of the button</param>
        /// <param name="imageRelativeUrl">The url for the image</param>
        /// <param name="onClickMethod">The method or script routine to call when the button is clicked.</param>
        /// <param name="htmlAttributes">Any attributes you want set on the tag. Use anonymous-type declaration for this: new{class=cssclass}</param>
        /// <returns></returns>
        public static string ImageButton(this HtmlHelper helper, string htmlName, string imageRelativeUrl, string onClickMethod) {
            return ImageButton(helper, "imageButton", imageRelativeUrl, onClickMethod, null);
        }

        /// <summary>
        /// A Simple image button you can use with javascript
        /// </summary>
        /// <param name="htmlName">Name of the button</param>
        /// <param name="buttonText">The text for the button face</param>
        /// <param name="onClickMethod">The method or script routine to call when the button is clicked.</param>
        /// <param name="htmlAttributes">Any attributes you want set on the tag. Use anonymous-type declaration for this: new{class=cssclass}</param>
        /// <returns></returns>
        public static string ImageButton(this HtmlHelper helper, string htmlName, string imageRelativeUrl, string onClickMethod, object htmlAttributes) {
            string buttonFormat = "<img name=\"{0}\" id=\"{0}\" src=\"{1}\" {2} onclick=\"{3}\" onmouseover=\"this.style.cursor='hand';\"/>";

            string atts = string.Empty;
            if (htmlAttributes != null)
                atts = htmlAttributes.ToAttributeList();
            
            string resolvedUrl = LinkExtensions.ResolveUrl(helper, imageRelativeUrl);

            return string.Format(buttonFormat, htmlName, resolvedUrl, atts, onClickMethod);
        }
        #endregion

        #region NavigateButton
        /// <summary>
        /// A Simple button you can use with javascript
        /// </summary>
        /// <param name="htmlName">Name of the button</param>
        /// <param name="buttonText">The text for the button face</param>
        /// <param name="navigateToUrl">The URL to navigate to onCLick.</param>
        /// <returns></returns>
        public static string NavigateButton(this HtmlHelper helper, string htmlName, string buttonText, string navigateToUrl) {
            return NavigateButton(helper, htmlName, buttonText, navigateToUrl, null);
        }
        /// <summary>
        /// A Simple button you can use with javascript
        /// </summary>
        /// <param name="htmlName">Name of the button</param>
        /// <param name="buttonText">The text for the button face</param>
        /// <param name="navigateToUrl">The URL to navigate to onCLick.</param>
        /// <param name="htmlAttributes">Any attributes you want set on the tag. Use anonymous-type declaration for this: new{class=cssclass}</param>
        /// <returns></returns>
        public static string NavigateButton(this HtmlHelper helper, string htmlName, string buttonText, string navigateToUrl, object htmlAttributes) {

            return Button(helper, htmlName, buttonText, "location.href='" + helper.Encode(navigateToUrl) + "'", htmlAttributes);
        }

        #endregion


    }
}
