﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.ComponentModel;
using System.Reflection;
using System.Web;
using System.Web.Compilation;
using System.IO;
using System.Collections.Specialized;
using System.Linq.Expressions;
using System.Data;

namespace System.Web.Mvc {
    public static class FormExtensions {

        #region Forms and Links

        #region Helper Classes and Methods
        public enum FormMethod {
            post,
            get
        }

        /// <summary>
        /// A class representing an HTML form. Access this object using Html.Form()
        /// </summary>
        public class SimpleForm : IDisposable {
            protected IHttpContext _context;

            bool wroteStartTag = false;
            bool wroteEndTag = false;
            string formFormat = "<form action=\"{0}\" method={1} {2}>";

            protected FormMethod _formMethod = FormMethod.post;
            protected object _htmlAttributes = null;
            protected string _formAction = string.Empty;

            protected SimpleForm() {

            }
            public SimpleForm(IHttpContext context, string formAction, FormMethod method, object htmlAttributes) {
                this._htmlAttributes = htmlAttributes;
                this._formMethod = method;
                this._context = context;
                this._formAction = formAction;
            }


            internal virtual string BuildFormOpenTag() {
                string attributeList = string.Empty;
                string result = string.Empty;
                string _method = Enum.GetName(typeof(FormMethod), _formMethod);
                if (_htmlAttributes != null)
                    attributeList = _htmlAttributes.ToAttributeList();
                
                result = string.Format(formFormat, _formAction, _method, attributeList);
               

                return result;
            }

            public void WriteStartTag() {
                if (!wroteStartTag) {
                    string formTag = BuildFormOpenTag();
                    _context.Response.Write(formTag);
                }
            }
            public void WriteEndTag() {
                if (!wroteEndTag)
                    _context.Response.Write("</form>");
            }

            public void Dispose() { WriteEndTag(); }
        }

        /// <summary>
        /// A class representing an HTML form. Access this object using Html.Form<T>()
        /// </summary>
        /// <typeparam name="T"></typeparam>
        public class MvcForm<T> : SimpleForm where T:Controller {

            public MvcForm(HtmlHelper helper, IHttpContext context, Expression<Action<T>> postAction, FormMethod method, object htmlAttributes) {
                MethodCallExpression call = postAction.Body as MethodCallExpression;

                //TODO- need to fix this
                this._context = context;
                this._formMethod = method;
                this._formAction = LinkExtensions.BuildUrlFromExpression(helper, postAction);

            }
            private MvcForm(IHttpContext context, string formAction, FormMethod method, object htmlAttributes) {

            }

        }

        #endregion

        /// <summary>
        /// Creates an HTML form opening tag based on the passed-in controller/action. Best use of this tag is within a using{} block: using Html.Form(...){}
        /// </summary>
        /// <typeparam name="T">The controller type</typeparam>
        /// <param name="postAction">The action method on the controller</param>
        /// <returns></returns>
        public static MvcForm<T> Form<T>(this HtmlHelper helper, Expression<Action<T>> postAction) where T : Controller {

            return Form<T>(helper, postAction,FormMethod.post, null);

        }

        /// <summary>
        /// Creates an HTML form opening tag based on the passed-in controller/action. Best use of this tag is within a using{} block: using Html.Form(...){}
        /// </summary>
        /// <typeparam name="T">The controller type</typeparam>
        /// <param name="postAction">The action method on the controller</param>
        /// <param name="method">POST or GET</param>
        /// <returns></returns>
        public static MvcForm<T> Form<T>(this HtmlHelper helper, Expression<Action<T>> postAction, FormMethod method) where T : Controller {

            return Form<T>(helper, postAction, method, null);

        }

        /// <summary>
        /// Creates an HTML form opening tag based on the passed-in controller/action. Best use of this tag is within a using{} block: using Html.Form(...){}
        /// </summary>
        /// <typeparam name="T">The controller type</typeparam>
        /// <param name="postAction">The action method on the controller</param>
        /// <param name="method">POST or GET</param>
        /// <param name="htmlAttributes">HTML Attributes to add to the form tag. Use an anonymous type for this (new {_class=formclass})</param>
        /// <returns></returns>
        public static MvcForm<T> Form<T>(this HtmlHelper helper, Expression<Action<T>> postAction, FormMethod method, object htmlAttributes) where T : Controller {

            MvcForm<T> form = new MvcForm<T>(helper, helper.ViewContext.HttpContext, postAction, method, htmlAttributes);
            form.WriteStartTag();
            return form;

        }

        /// <summary>
        /// Creates an HTML form opening tag based on the passed-in controller/action. Best use of this tag is within a using{} block: using Html.Form(...){}
        /// </summary>
        /// <param name="controllerName">The name of the controller</param>
        /// <param name="actionName">The name of the action</param>
        public static SimpleForm Form(this HtmlHelper helper, string actionName, string controllerName) {
            return Form(helper, actionName, controllerName, "", FormMethod.post, null);
        }
        /// <summary>
        /// Creates an HTML form opening tag based on the passed-in controller/action. Best use of this tag is within a using{} block: using Html.Form(...){}
        /// </summary>
        /// <param name="controllerName">The name of the controller</param>
        /// <param name="actionName">The name of the action</param>
        /// <param name="method">POST or GET</param>
        public static SimpleForm Form(this HtmlHelper helper, string actionName, string controllerName, FormMethod method) {
            return Form(helper, actionName, controllerName, "", method, null);
        }
        /// <summary>
        /// Creates an HTML form opening tag based on the passed-in controller/action. Best use of this tag is within a using{} block: using Html.Form(...){}
        /// </summary>
        /// <param name="controllerName">The name of the controller</param>
        /// <param name="actionName">The name of the action</param>
        /// <param name="htmlAttributes">HTML Attributes to add to the form tag. Use an anonymous type for this (new {_class=formclass})</param>
        public static SimpleForm Form(this HtmlHelper helper, string actionName, string controllerName, object htmlAttributes) {
            return Form(helper, actionName, controllerName, "", FormMethod.post, htmlAttributes);
        }

        /// <summary>
        /// Creates an HTML form opening tag based on the passed-in controller/action. Best use of this tag is within a using{} block: using Html.Form(...){}
        /// </summary>
        /// <param name="controllerName">The name of the controller</param>
        /// <param name="actionName">The name of the action</param>
        /// <param name="htmlAttributes">HTML Attributes to add to the form tag. Use an anonymous type for this (new {_class=formclass})</param>
        public static SimpleForm Form(this HtmlHelper helper, string actionName, string controllerName, FormMethod method, object htmlAttributes) {
            return Form(helper, actionName, controllerName, "", method, htmlAttributes);
        }


        /// <summary>
        /// Creates an HTML form opening tag based on the passed-in controller/action. Best use of this tag is within a using{} block: using Html.Form(...){}
        /// </summary>
        /// <param name="controllerName">The name of the controller</param>
        /// <param name="actionName">The name of the action</param>
        /// <param name="queryString">Any query string values to pass to the controller</param>
        /// <param name="method">POST or GET</param>
        /// <param name="htmlAttributes">HTML Attributes to add to the form tag. Use an anonymous type for this (new {_class=formclass})</param>
        /// <returns></returns>
        public static SimpleForm Form(this HtmlHelper helper, string actionName, string controllerName, string queryString, FormMethod method, object htmlAttributes) {

            if (!queryString.StartsWith("?") && !String.IsNullOrEmpty(queryString))
                queryString = "?" + queryString;

            UrlHelper h = new UrlHelper(helper.ViewContext);
            if (controllerName.EndsWith("Controller"))
                controllerName = controllerName.Replace("Controller", "");

            string formAction =LinkExtensions.BuildUrl(helper,controllerName,actionName,queryString);

            SimpleForm form = new SimpleForm(helper.ViewContext.HttpContext, formAction, method, htmlAttributes);
            form.WriteStartTag();
            return form;

        }

        #endregion


    }
}
